from collections import OrderedDict

from django.conf import settings
from django.db import models
from django.utils.functional import cached_property

from register.models.attendee import Attendee


class Meal(models.Model):
    MEALS = OrderedDict((
        ('breakfast', 'Breakfast'),
        ('brunch', 'Brunch'),
        ('lunch', 'Lunch'),
        ('dinner', 'Dinner'),
    ))

    date = models.DateField(db_index=True)
    meal = models.CharField(max_length=16)

    @cached_property
    def form_name(self):
        return '{}_{}'.format(self.meal, self.date.isoformat())

    @cached_property
    def conference_dinner(self):
        return (self.meal == 'dinner'
            and self.date == settings.DEBCONF_CONFERENCE_DINNER_DAY)

    def __str__(self):
        return '{}: {}'.format(self.date.isoformat(), self.meal)

    class Meta:
        ordering = ['date']
        unique_together = ('date', 'meal')


class Allergen(models.Model):
    ALLERGENS = OrderedDict((
        ("gluten", "Cereals containing gluten"),
        ("crustacean", "Crustaceans"),
        ("eggs", "Eggs"),
        ("fish", "Fish"),
        ("peanut", "Peanuts"),
        ("soybean", "Soybeans"),
        ("milk", "Milk"),
        ("nut", "Nuts"),
        ("celery", "Celery"),
        ("mustard", "Mustard"),
        ("seasame", "Sesame seeds"),
        ("sulphur", "Sulphur dioxide and sulphites"),
        ("lupin", "Lupin"),
        ("mollusc", "Molluscs"),
    ))
    name = models.CharField(max_length=16, unique=True)
    description = models.CharField(max_length=32, unique=True)

    def __str__(self):
        return self.description


class Food(models.Model):
    DIETS = OrderedDict((
        ('', 'I will be happy to eat whatever is provided'),
        ('vegetarian', "I am lacto-ovo vegetarian, don't provide "
                       "meat/fish for me"),
        ('vegan', "I am strict vegetarian (vegan), don't provide any "
                  "animal products for me"),
        ('gluten_free', 'I require gluten-free food'),
        ('other', 'Other, described below'),
    ))

    attendee = models.OneToOneField(Attendee, related_name='food',
                                    on_delete=models.CASCADE)

    meals = models.ManyToManyField(Meal)
    diet = models.CharField(max_length=16, blank=True)
    allergens = models.ManyToManyField(Allergen)
    special_diet = models.TextField(blank=True)

    def __str__(self):
        return 'Attendee <{}>'.format(self.attendee.user.username)

    @cached_property
    def meals_by_day(self):
        meal_order = list(Meal.MEALS.keys())
        meals = sorted(
            self.meals.all(),
            key=lambda meal: (meal.date, meal_order.index(meal.meal)))

        by_day = OrderedDict()
        for meal in meals:
            by_day.setdefault(meal.date, []).append(meal)
        return by_day



# Generated by Django 4.2.23 on 2026-02-10 19:19

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bursary", "0006_bursary_budget"),
    ]

    operations = [
        migrations.AddField(
            model_name="bursary",
            name="reimbursement_currency",
            field=models.CharField(
                choices=[
                    ("AED", "AED (UAE Dirham)"),
                    ("AFN", "AFN (Afghani)"),
                    ("ALL", "ALL (Lek)"),
                    ("AMD", "AMD (Armenian Dram)"),
                    ("AOA", "AOA (Kwanza)"),
                    ("ARS", "ARS (Argentine Peso)"),
                    ("AUD", "AUD (Australian Dollar)"),
                    ("AWG", "AWG (Aruban Florin)"),
                    ("AZN", "AZN (Azerbaijan Manat)"),
                    ("BAM", "BAM (Convertible Mark)"),
                    ("BBD", "BBD (Barbados Dollar)"),
                    ("BDT", "BDT (Taka)"),
                    ("BHD", "BHD (Bahraini Dinar)"),
                    ("BIF", "BIF (Burundi Franc)"),
                    ("BMD", "BMD (Bermudian Dollar)"),
                    ("BND", "BND (Brunei Dollar)"),
                    ("BOB", "BOB (Boliviano)"),
                    ("BOV", "BOV (Mvdol)"),
                    ("BRL", "BRL (Brazilian Real)"),
                    ("BSD", "BSD (Bahamian Dollar)"),
                    ("BTN", "BTN (Ngultrum)"),
                    ("BWP", "BWP (Pula)"),
                    ("BYN", "BYN (Belarusian Ruble)"),
                    ("BZD", "BZD (Belize Dollar)"),
                    ("CAD", "CAD (Canadian Dollar)"),
                    ("CDF", "CDF (Congolese Franc)"),
                    ("CHE", "CHE (WIR Euro)"),
                    ("CHF", "CHF (Swiss Franc)"),
                    ("CHW", "CHW (WIR Franc)"),
                    ("CLF", "CLF (Unidad de Fomento)"),
                    ("CLP", "CLP (Chilean Peso)"),
                    ("CNY", "CNY (Yuan Renminbi)"),
                    ("COP", "COP (Colombian Peso)"),
                    ("COU", "COU (Unidad de Valor Real)"),
                    ("CRC", "CRC (Costa Rican Colon)"),
                    ("CUP", "CUP (Cuban Peso)"),
                    ("CVE", "CVE (Cabo Verde Escudo)"),
                    ("CZK", "CZK (Czech Koruna)"),
                    ("DJF", "DJF (Djibouti Franc)"),
                    ("DKK", "DKK (Danish Krone)"),
                    ("DOP", "DOP (Dominican Peso)"),
                    ("DZD", "DZD (Algerian Dinar)"),
                    ("EGP", "EGP (Egyptian Pound)"),
                    ("ERN", "ERN (Nakfa)"),
                    ("ETB", "ETB (Ethiopian Birr)"),
                    ("EUR", "EUR (Euro)"),
                    ("FJD", "FJD (Fiji Dollar)"),
                    ("FKP", "FKP (Falkland Islands Pound)"),
                    ("GBP", "GBP (Pound Sterling)"),
                    ("GEL", "GEL (Lari)"),
                    ("GHS", "GHS (Ghana Cedi)"),
                    ("GIP", "GIP (Gibraltar Pound)"),
                    ("GMD", "GMD (Dalasi)"),
                    ("GNF", "GNF (Guinean Franc)"),
                    ("GTQ", "GTQ (Quetzal)"),
                    ("GYD", "GYD (Guyana Dollar)"),
                    ("HKD", "HKD (Hong Kong Dollar)"),
                    ("HNL", "HNL (Lempira)"),
                    ("HTG", "HTG (Gourde)"),
                    ("HUF", "HUF (Forint)"),
                    ("IDR", "IDR (Rupiah)"),
                    ("ILS", "ILS (New Israeli Sheqel)"),
                    ("INR", "INR (Indian Rupee)"),
                    ("IQD", "IQD (Iraqi Dinar)"),
                    ("IRR", "IRR (Iranian Rial)"),
                    ("ISK", "ISK (Iceland Krona)"),
                    ("JMD", "JMD (Jamaican Dollar)"),
                    ("JOD", "JOD (Jordanian Dinar)"),
                    ("JPY", "JPY (Yen)"),
                    ("KES", "KES (Kenyan Shilling)"),
                    ("KGS", "KGS (Som)"),
                    ("KHR", "KHR (Riel)"),
                    ("KMF", "KMF (Comorian Franc)"),
                    ("KPW", "KPW (North Korean Won)"),
                    ("KRW", "KRW (Won)"),
                    ("KWD", "KWD (Kuwaiti Dinar)"),
                    ("KYD", "KYD (Cayman Islands Dollar)"),
                    ("KZT", "KZT (Tenge)"),
                    ("LAK", "LAK (Lao Kip)"),
                    ("LBP", "LBP (Lebanese Pound)"),
                    ("LKR", "LKR (Sri Lanka Rupee)"),
                    ("LRD", "LRD (Liberian Dollar)"),
                    ("LSL", "LSL (Loti)"),
                    ("LYD", "LYD (Libyan Dinar)"),
                    ("MAD", "MAD (Moroccan Dirham)"),
                    ("MDL", "MDL (Moldovan Leu)"),
                    ("MGA", "MGA (Malagasy Ariary)"),
                    ("MKD", "MKD (Denar)"),
                    ("MMK", "MMK (Kyat)"),
                    ("MNT", "MNT (Tugrik)"),
                    ("MOP", "MOP (Pataca)"),
                    ("MRU", "MRU (Ouguiya)"),
                    ("MUR", "MUR (Mauritius Rupee)"),
                    ("MVR", "MVR (Rufiyaa)"),
                    ("MWK", "MWK (Malawi Kwacha)"),
                    ("MXN", "MXN (Mexican Peso)"),
                    ("MXV", "MXV (Mexican Unidad de Inversion (UDI))"),
                    ("MYR", "MYR (Malaysian Ringgit)"),
                    ("MZN", "MZN (Mozambique Metical)"),
                    ("NAD", "NAD (Namibia Dollar)"),
                    ("NGN", "NGN (Naira)"),
                    ("NIO", "NIO (Cordoba Oro)"),
                    ("NOK", "NOK (Norwegian Krone)"),
                    ("NPR", "NPR (Nepalese Rupee)"),
                    ("NZD", "NZD (New Zealand Dollar)"),
                    ("OMR", "OMR (Rial Omani)"),
                    ("PAB", "PAB (Balboa)"),
                    ("PEN", "PEN (Sol)"),
                    ("PGK", "PGK (Kina)"),
                    ("PHP", "PHP (Philippine Peso)"),
                    ("PKR", "PKR (Pakistan Rupee)"),
                    ("PLN", "PLN (Zloty)"),
                    ("PYG", "PYG (Guarani)"),
                    ("QAR", "QAR (Qatari Rial)"),
                    ("RON", "RON (Romanian Leu)"),
                    ("RSD", "RSD (Serbian Dinar)"),
                    ("RUB", "RUB (Russian Ruble)"),
                    ("RWF", "RWF (Rwanda Franc)"),
                    ("SAR", "SAR (Saudi Riyal)"),
                    ("SBD", "SBD (Solomon Islands Dollar)"),
                    ("SCR", "SCR (Seychelles Rupee)"),
                    ("SDG", "SDG (Sudanese Pound)"),
                    ("SEK", "SEK (Swedish Krona)"),
                    ("SGD", "SGD (Singapore Dollar)"),
                    ("SHP", "SHP (Saint Helena Pound)"),
                    ("SLE", "SLE (Leone)"),
                    ("SOS", "SOS (Somali Shilling)"),
                    ("SRD", "SRD (Surinam Dollar)"),
                    ("SSP", "SSP (South Sudanese Pound)"),
                    ("STN", "STN (Dobra)"),
                    ("SVC", "SVC (El Salvador Colon)"),
                    ("SYP", "SYP (Syrian Pound)"),
                    ("SZL", "SZL (Lilangeni)"),
                    ("THB", "THB (Baht)"),
                    ("TJS", "TJS (Somoni)"),
                    ("TMT", "TMT (Turkmenistan New Manat)"),
                    ("TND", "TND (Tunisian Dinar)"),
                    ("TOP", "TOP (Pa’anga)"),
                    ("TRY", "TRY (Turkish Lira)"),
                    ("TTD", "TTD (Trinidad and Tobago Dollar)"),
                    ("TWD", "TWD (New Taiwan Dollar)"),
                    ("TZS", "TZS (Tanzanian Shilling)"),
                    ("UAH", "UAH (Hryvnia)"),
                    ("UGX", "UGX (Uganda Shilling)"),
                    ("USD", "USD (US Dollar)"),
                    ("USN", "USN (US Dollar (Next day))"),
                    ("UYI", "UYI (Uruguay Peso en Unidades Indexadas (UI))"),
                    ("UYU", "UYU (Peso Uruguayo)"),
                    ("UYW", "UYW (Unidad Previsional)"),
                    ("UZS", "UZS (Uzbekistan Sum)"),
                    ("VED", "VED (Bolívar Soberano)"),
                    ("VES", "VES (Bolívar Soberano)"),
                    ("VND", "VND (Dong)"),
                    ("VUV", "VUV (Vatu)"),
                    ("WST", "WST (Tala)"),
                    ("XAF", "XAF (CFA Franc BEAC)"),
                    ("XCD", "XCD (East Caribbean Dollar)"),
                    ("XOF", "XOF (CFA Franc BCEAO)"),
                    ("XPF", "XPF (CFP Franc)"),
                    ("YER", "YER (Yemeni Rial)"),
                    ("ZAR", "ZAR (Rand)"),
                    ("ZMW", "ZMW (Zambian Kwacha)"),
                    ("ZWG", "ZWG (Zimbabwe Gold)"),
                ],
                default=settings.DEBCONF_BURSARY_CURRENCY,
                max_length=3,
            ),
            preserve_default=False,
        ),
    ]



from django.conf import settings

from zammad_py import ZammadAPI

from bursary.models import Bursary
from register.forms.misc import MiscForm
from register.models.attendee import Attendee
from register.views.core import RegisterStep


class MiscView(RegisterStep):
    title = 'Anything Else?'
    form_class = MiscForm

    def get_initial(self):
        user = self.request.user
        initial = {
            'notes': self.request.user.attendee.notes
        }
        try:
            bursary = user.bursary
        except Bursary.DoesNotExist:
            pass
        else:
            for field in bursary._meta.get_fields():
                if field.is_relation:
                    continue
                initial[field.name] = getattr(bursary, field.name)
        return initial

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['contributions'] = self.request_contributions()
        return kwargs

    def request_contributions(self):
        user = self.request.user
        try:
            bursary = user.bursary
        except Bursary.DoesNotExist:
            bursary = Bursary()
        return (
            settings.DEBCONF_REVIEW_FREE_ATTENDEES
            and not user.attendee.billable()
            and not bursary.potential_bursary())

    def create_or_update_zammad_ticket(self, user, notes):
        if not settings.ZAMMAD_URL:
            return

        client = ZammadAPI(url=settings.ZAMMAD_URL, http_token=settings.ZAMMAD_TOKEN)
        # Update the first existing ticket, if we find any
        for ticket in client.ticket.search(
            f"customer.email:{user.email} "
            f"AND tags:registration_notes "
            f'AND group.name:"Users::Registration"'
        ):
            if ticket["state"] != "open":
                client.ticket.update(ticket["id"], params={"state": "open"})
            client.ticket_article.create(params={
                "ticket_id": ticket["id"],
                "subject": f"Registration notes for {user.username}",
                "body": f"Registration was updated. The notes are now:\n{notes}",
                "content_type": "text/plain",
                "type": "note",
                "internal": False,
            })
            return
        ticket = client.ticket.create(params={
            "title": f"Registration notes for {user.username}",
            "group": "Users::Registration",
            "customer": user.email,
            "article": {
                "body": f"Registration notes:\n{notes}",
                "content_type": "text/plain",
                "type": "note",
                "internal": False,
            },
        })
        client.ticket_tag.add(ticket["id"], "registration_notes")

    def form_valid(self, form):
        user = self.request.user
        data = form.cleaned_data
        bursary_data = data.copy()
        attendee_data = {'notes': bursary_data.pop('notes')}
        bursary_data.update({
            'request_food': False,
            'request_accommodation': False,
            'request_travel': False,
            'request_expenses': False,
        })

        user.attendee  # We should never be creating, here
        if user.attendee.notes != attendee_data['notes']:
            self.create_or_update_zammad_ticket(user, attendee_data['notes'])
        user.attendee = Attendee.objects.update_or_create(
            user=user, defaults=attendee_data)[0]

        if self.request_contributions():
            user.bursary = Bursary.objects.update_or_create(
                user=user, defaults=bursary_data)[0]

        return super().form_valid(form)



# Generated by Django 4.2.23 on 2026-02-10 00:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("register", "0019_alter_attendee_permissions"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="attendee",
            name="confirm_covid_tests",
        ),
        migrations.RemoveField(
            model_name="attendee",
            name="vaccinated",
        ),
        migrations.RemoveField(
            model_name="attendee",
            name="vaccination_notes",
        ),
    ]



# Generated by Django 4.2.23 on 2026-02-10 01:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("register", "0020_remove_covid19_form"),
    ]

    operations = [
        migrations.CreateModel(
            name="Allergen",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=16, unique=True)),
                ("description", models.CharField(max_length=32, unique=True)),
            ],
        ),
        migrations.AddField(
            model_name="food",
            name="allergens",
            field=models.ManyToManyField(to="register.allergen"),
        ),
    ]



# Cribbed from: https://github.com/stefanor/django-historical-currencies/blob/main/historical_currencies/choices.py

from iso4217 import Currency


def iter_choices(exclude_special=True):
    """Iterator that produces a list of (unsorted) currency choices"""
    for currency in Currency:
        # Generally X currencies aren't currencies (except the ones that are)
        if (
            exclude_special
            and currency.code.startswith("X")
            and currency.code not in ("XAF", "XCD", "XOF", "XPF")
        ):
            continue

        yield (
            currency.code,
            f"{currency.code} ({currency.currency_name})",
        )


def currency_choices(**kwargs):
    """A list of CURRENCY: Descriptive Name for use in Django choices"""
    return sorted(list(iter_choices(**kwargs)))
